#!/bin/sh
# Cygwin Boost package build script.

# There are some problems with Boost DLLs for Cygwin. Mostly the fact that it
# does not generate import libraries. So there will be only static Boost libs 
# until the problem gets sorted out.
#build_opts="release <threading>multi <runtime-link>dynamic/static"
build_opts="release <threading>multi <runtime-link>static <define>BOOST_POSIX=1 <optimization>speed"
python_ver="2.4"
package_ver="1.33.1-2"
source_file="boost_1_33_1.tar.bz2"
source_dir="boost_1_33_1"
cwd=`pwd`
install_dir="${cwd}/fake-root"
this_script="$0"

# Before we do anything else we build Cygwin -src package.
echo -n "Building boost-${package_ver}-src.tar.bz2..."
if test ! -f "stamp-src" ; then
    rm -f "boost-${package_ver}-src.tar.bz2"
    rm -rf "${cwd-nonexistentdir}/boost-${package_ver}"
    rm -f boost-devel/*~
    rm -f libboost/*~

    mkdir "boost-${package_ver}" \
    && cp -pRf -t "boost-${package_ver}" "${this_script}" \
        boost-devel setup.hint boost.README "${source_file}" \
	patches \
    && tar -cj -f "boost-${package_ver}-src.tar.bz2" "boost-${package_ver}" \
    && rm -rf "${cwd-nonexistentdir}/boost-${package_ver}" \
    && touch stamp-src \
    || exit 1
fi
echo "done."

# Unpack the source.
echo -n "Extracting source..."
if test ! -f stamp-unpacked ; then
    rm -f stamp-patched
    tar -xj --unlink-first --recursive-unlink -f "${source_file}" \
    && touch stamp-unpacked \
    || exit 1
fi
echo "done."

# Patch the source with Cygwin specific patches.
#echo "Patching source..."
#if test ! -f stamp-patched ; then
#    for p in "${cwd}/patches/patch-operations_posix_windows.cpp.diff" ; do
#	echo "Applying patches..."
#	(cd "${source_dir}" \
#	 && patch `awk -- 'NR == 2 { print $2 }' < "${p}"` "${p}") \
#	|| exit 1
#    done
#    touch stamp-patched || exit 1
#fi
#echo "done."

# Build bjam.
echo -n "Building bjam..."
if test ! -e "${source_dir}/tools/build/jam_src/bin.cygwinx86/bjam.exe" ; then
    (cd "${source_dir}/tools/build/jam_src" && sh ./build.sh >/dev/null) \
    || exit 1
fi
echo "done."

# Build and install the Boost into a fake root.
(
    cd "${source_dir}"
    chmod +x ./tools/build/jam_src/bin.cygwinx86/bjam.exe
    ./tools/build/jam_src/bin.cygwinx86/bjam -d1 \
        -sTOOLS=gcc \
        -sBUILD="${build_opts}" \
        -sPYTHON_VERSION="${python_ver}" \
        --builddir=./build-gcc \
        --stagedir=./stage-gcc \
        --prefix="${install_dir}/usr" \
        --sysconfdir="${install_dir}/etc" \
        --libexecdir="${install_dir}/usr/sbin" \
        --localstatedir="${install_dir}/var" \
        --datadir="${install_dir}/usr/share" \
        --mandir="${install_dir}/usr/share/man" \
        --infodir="${install_dir}/usr/share/info" \
        --without-test \
        --layout=versioned \
        install
) # || exit 1 # Cygwin doesn't have wchar_t support and we get $? != 0.

# No DLLs for Cygwin. See comment on top.
# Move Boost's .dlls into bin directory.
#echo -n "Moving Boost's .dlls into bin directory..."
#(
#     mkdir -p -- "${install_dir}/usr/bin"
#     (cd "${install_dir}/usr/lib" \
#     && strip libboost_*.dll \
#     && mv -f -t ../bin libboost_*.dll)
# ) || exit 1
# echo "done."

# Copy boost.README to its place before packaging.
echo -n "Copying boost.README..."
mkdir -p -- "${install_dir}/usr/share/doc/Cygwin" \
&& cp -pf -t "${install_dir}/usr/share/doc/Cygwin" boost.README \
|| exit 1
echo "done."

# Copy Boost's docs into place.
echo -n "Copying Boost's docs into place..."
docs_dir="${install_dir}/usr/share/doc/boost-${package_ver}"
rm -rf "${docs_dir}"
(mkdir -p -- "${docs_dir}" \
    && \
    (  if test ! -e "${cwd}/stamp-chmod" ; then \
    find "${source_dir}" -type f -exec chmod 0444 {} \; \
    && find "${source_dir}" -type d -exec chmod 0755 {} \; \
    && find "${source_dir}" -name ".cvsignore" -delete \
    && touch stamp-chmod ; \
    fi ) \
    && \
    (  cd "${source_dir}" \
    && find . -name "*.htm" -o -name "*.html"  -o -name "*.css" \
    -o -name "*.jp*g" -o -name "*.png" -o -name "*.gif" \
    -o -name "README" -o -name  "LICENSE_1_0.txt" \
    | tar -c -T - -f - | tar -x --unlink-first --recursive-unlink -C "${docs_dir}" -f - ) \
    && \
    (  cd "${source_dir}/libs" \
    && find . -name "example" -o -name "examples" -o -name "*example*.cpp" \
    -o -name "*example*.c" -o -name "*example*.hpp" | uniq \
    | grep -v "assign/test" | grep -v "iterator/test" \
    | grep -v "range/test" \
    | tar -c -T - -f - | tar -x --unlink-first --recursive-unlink -C "${docs_dir}/libs" -f - )
) || exit 1
echo "done."

# Fix permissions.
echo -n "Fixing permissions..."
(cd "${install_dir}/usr/lib" && chmod u+rw-x,go+r-wx libboost_*.a) || exit 1
echo "done."

# Build boost package (main package).
echo -n "Building boost-${package_ver}.tar.bz2..."
rm -f "boost-${package_ver}.tar.bz2"
tar -cj -C "${install_dir}" -f "boost-${package_ver}.tar.bz2" \
    usr/share || exit 1
echo "done."

# No DLLs for Cygwin. See comment on top.
# Build libboost package (DLLs).
#echo -n "Building libboost-${package_ver}.tar.bz2..."
#rm -f "libboost-${package_ver}.tar.bz2"
#tar -cj -C "${install_dir}" -f "libboost-${package_ver}.tar.bz2" \
#    usr/bin || exit 1
#echo "done."

# Build boost-devel package (static libs and headers).
echo -n "Building boost-devel-${package_ver}.tar.bz2..."
rm -f "boost-devel-${package_ver}.tar.bz2"
tar -cj -C "${install_dir}" -f "boost-devel-${package_ver}.tar.bz2" \
    usr/lib usr/include || exit 1
echo "done."

# Move the generated packages into proper directories, 
# besides their setup.hint files.
# However, doing this complicates building of -src, so we'd better not do it.
#echo "Moving built packages to proper subdirs..."
#mv -vf -t libboost/ "libboost-${package_ver}.tar.bz2"
#mv -vf -t boost-devel/ "boost-devel-${package_ver}.tar.bz2"
